import subprocess
from abc import ABC, abstractmethod
from dataclasses import dataclass
from time import sleep


@dataclass(frozen=True)
class MessageDTO:
    type_msg: str
    message: str
    detail: str


class Command(ABC):
    @property
    def command(self):
        return self.__command

    @command.setter
    def command(self, cmd: list):
        self.__command = cmd

    @abstractmethod
    def _stdout_parser(self, stdout: str) -> list[MessageDTO]:
        raise NotImplementedError

    def _execute(self, command: list) -> tuple[str, str]:
        result = subprocess.run(
            command,
            capture_output=True,
            check=True,
            text=True,
        )
        return result.stdout, result.stderr

    def get_alerts(self) -> list[MessageDTO]:
        stdout, stderr = self._execute(command=self.command)
        if stderr:
            return [
                MessageDTO(
                    type_msg="Error",
                    message="Free command execution error",
                    detail=stderr,
                )
            ]
        alerts = self._stdout_parser(stdout=stdout)
        return alerts


class FreeMem(Command):
    def __init__(
        self,
    ) -> None:
        self.command = ["free", "-h"]

    def _stdout_parser(self, stdout: str) -> list[MessageDTO]:
        memory = stdout.split()[9]
        memory_units = memory[-2:]
        memory = memory[:-2].replace(",", ".")
        memory_value = float(memory) if "." in memory else int(memory)
        return [
            MessageDTO(
                type_msg="Status",
                message="free mem",
                detail=f"{memory_value} {memory_units}",
            )
        ]


class FreeHdd(Command):
    def __init__(self) -> None:
        self.command = ["df", "-h"]

    def _stdout_parser(self, stdout) -> list[MessageDTO]:
        return []


def send_alert(alerts: list) -> None:
    pass
    # print(*alerts, sep="\n")


# def shell_command_result(command: list) -> tuple[str, str]:
#     process = subprocess.run(
#         command,
#         capture_output=True,
#         check=True,
#         text=True,
#     )
#     return process.stdout, process.stderr
#
#
# def get_free_mem() -> tuple[int | float, str]:
#     stdout, stderr = shell_command_result(command=["free", "-h"])
#     if stderr:
#         return
#
#         return memory_value, memory_units
#
#
# def get_hdd_free_mem() -> list[tuple[str, str]]:
#     stdout, stderr = shell_command_result(command=["df", "-h"])
#     rows = [row.split() for row in stdout.split("\n")]
#     mount_points = [(row[5], row[4]) for row in rows[1:] if row]
#     return mount_points
#
#
def main_loop() -> None:
    free_mem = FreeMem()
    while True:
        stats = free_mem.get_alerts()

        stats += []
        # print(stats)
        sleep(1)


if __name__ == "__main__":
    main_loop()
